/*
 *  Copyright (C) 2007  FUJITSU LIMITED
 */
/* $Id: darray_string.c,v 1.4 2007/05/07 16:06:58 tachino Exp $ */

#include <regex.h>

#include "string_hash.h"
#include "darray_string.h"


DEFINE_DARRAY_ALLOC(string, xstrdup, free);

static const char *sep_default = " \t\n\r";

darray_string *split(const char *_line, const char *sep, int limit)
{
	darray_string *da;
	char *line = xstrdup(_line);
	char * const linebase = line;

	da = darray_string_alloc();

	if (sep == NULL)
		sep = sep_default;

	while (*line) {
		int len;
		char *elem;
		char c;

		line += strspn(line, sep);
		if (!*line)
			break;

		if (limit && da->length == limit) {
			darray_push(da, line);
			break;
		}

		elem = line;
		len = strcspn(elem, sep);
		line += len;

		c = *line;
		*line = '\0';
		darray_push(da, elem);
		*line = c;
	}

	free(linebase);
	return da;
}

darray_string *split_regex(const char *_line, const char *sep, int limit)
{
	darray_string *da;
	char *line = xstrdup(_line);
	char * const linebase = line;
	int length = strlen(line);
	regex_t regex;
	regmatch_t rm;

	da = darray_string_alloc();

	if (regcomp(&regex, sep, REG_EXTENDED) != 0)
		die("regcomp");

	/* punch holes */
	while (*line) {
		if (regexec(&regex, line, 1, &rm, 0) != 0)
			break;

		memset(line + rm.rm_so, 0, rm.rm_eo - rm.rm_so);
		line += rm.rm_eo;
	}

	for (line = linebase; line < linebase + length; line++) {
		if (!*line)
			continue;

		if (limit && da->length == limit) {
			darray_push(da, line);
			break;
		}

		darray_push(da, line);
		line += strlen(line);
	}

	free(linebase);
	regfree(&regex);
	return da;
}

darray_uint *darray_string_hash_array(darray_string *da_str)
{
	darray_uint *da_uint = darray_uint_alloc();
	string *s;

	darray_foreach(s, da_str)
		darray_push(da_uint, string_hash((const unsigned char *)*s));

	return da_uint;
}
