#ifndef DISKDUMPCMN_H
#define DISKDUMPCMN_H

/* $Id: diskdumpcmn.h,v 1.25 2007/05/31 15:33:47 akira Exp $ */

#include <stdbool.h>

#define DUMP_PARTITION_SIGNATURE	"diskdump"
#define PART_SIG_LEN			(sizeof(DUMP_PARTITION_SIGNATURE) - 1)
#define DISK_DUMP_SIGNATURE		"DISKDUMP"
#define DUMP_SIG_LEN			(sizeof(DISK_DUMP_SIGNATURE) - 1)
#define ALREADY_SAVED_SIGNATURE		"DMPSAVED"
#define SAVE_SIG_LEN			(sizeof(ALREADY_SAVED_SIGNATURE) - 1)
#define SWAP_PARTITION_SIGNATURE	"SWAPSPACE2"
#define SWAP_SIG_LEN			(sizeof(SWAP_PARTITION_SIGNATURE) - 1)

#define FAILED		-1
#define SUCCEEDED	0

#define Err(x, ...)	fprintf(stderr, "%s: " x, cmdname, ## __VA_ARGS__)
#define Syserr(x, ...)	fprintf(stderr, "%s: unable to " x ": %s\n",\
				cmdname, ## __VA_ARGS__, strerror(errno))
#define Info(x...)	do { if (verbose) fprintf(stderr, x); } while (0)

#define _PROC_DISKDUMP	"/proc/diskdump"

#define DATE_TIME_FORMAT	"YYYY-mm-dd-HH:MM"
#define CURRENT_DATE_TIME	time(NULL)

#define min(x, y)	((x) > (y) ? (y) : (x))
#define max(x, y)	((x) > (y) ? (x) : (y))

#define	countof(array)	(sizeof(array) / sizeof((array)[0]))

#define die(x, ...)	die_func(__FILE__, __LINE__, x, ## __VA_ARGS__)

#define xmalloc(_sz)							\
	({								\
	 	typeof(_sz) sz = _sz;					\
	 	void *p = malloc(sz);					\
									\
		if (p == NULL)						\
			die("malloc(%ld) failed", (long)sz);		\
		p;							\
	})

#define xcalloc(_n, _sz)						\
	({								\
	 	typeof(_sz) sz = _sz;					\
	 	typeof(_n) n = _n;					\
	 	void *p = calloc(n, sz);				\
									\
		if (p == NULL)						\
			die("calloc(%d, %ld) failed", n, (long)sz);	\
		p;							\
	})

#define xstrdup(org)							\
	({								\
	 	char *dup = strdup(org);				\
									\
		if (dup == NULL)					\
			die("strdup(%ld) failed", (long)strlen(org));	\
		dup;							\
	})

#define xasprintf(fmt, ...)						\
	({								\
	 	char *p;						\
	 	if (asprintf(&p, fmt, ## __VA_ARGS__) < 0)		\
	 		die("asprintf(%s) failed", fmt);		\
	 	p;							\
	})

#define xvasprintf(fmt, ap)						\
	({								\
	 	char *p;						\
	 	if (vasprintf(&p, fmt, ap) < 0)				\
	 		die("asprintf(%s) failed", fmt);		\
	 	p;							\
	})

#define memdup(src) (typeof(src))({					\
	typeof(src) dst;						\
									\
	dst = xmalloc(sizeof(*src));					\
	memcpy(dst, src, sizeof(*src));					\
									\
	dst; })

#include <stdint.h>

typedef unsigned long long u64;
typedef unsigned long addr_t;		/* The type of kernel address */

static inline bool bad_addr(addr_t addr)
{
	return addr == (addr_t)(-1L);
}

/*
 * Version number is expressed by "i.j.k".
 * i: major version number
 * j: minor version number
 * k: micro version number
 */
struct version_form {
	int	major;
	int	minor;
	int	micro;
};


extern const char *cmdname;


void	 usage(void)	__attribute__ ((noreturn));
void	 version(void)	__attribute__ ((noreturn));
u64	 scan_param_proc(const char *param_name);
int	 convert_version_number(struct version_form *ver, const char *ver_str);
void	 progress_bar(const char *item, int symbol, off_t now, off_t max);
void	 progress_bar_post(const char *item, int symbol, off_t now, off_t max);
int	 execute_script(const char *script, const char *args);
int	 rm_dir(const char *path);
char	*convert_ip_addr(uint32_t ip);
bool	 is_swapon(void);
int	 is_mounted(const char *device_name);
int	 is_block_device(const char *device_name);
char	*get_date_time_str(const time_t);
int	 diff_days(time_t, time_t);
bool	 same_file(const char *src, const char *dst);
bool	 does_exist(const char *file);
void	*read_alloc(int fd, size_t len);
int	 get_page_size(void);
bool	 dump_signature_is_good(const void *signature);
bool	 part_signature_is_good(const void *signature);
bool	 swap_signature_is_good(const void *signature);
char	*kernel_path(const char *kernel);
void	 die_func(const char *filename, int lineno, const char *fmt, ...) __attribute__((noreturn)) __attribute__((format (printf, 3, 4)));
bool	 start_with(const char *str, const char *prefix);
char	*chop(char *line, const char *sep);
bool	 is_later(const char *filename, time_t);
bool	 is_super_user(void);
bool	 same_fs(const char *src, const char *dst);

#endif
