#ifndef DUMPFILE_H
#define DUMPFILE_H

/* $Id: dumpfile.h,v 1.6 2007/04/27 22:01:13 tachino Exp $ */

#include "darray.h"
#include "diskdumpelf.h"
#include "dumpheader.h"
#include "systemmap.h"

#define EXISTING	0
#define NEW		1

#define DUMP_TYPE_ELF		0
#define DUMP_TYPE_COMPRESSED	1

typedef Elf_Phdr	elfphdr;
EXPORT_DARRAY(elfphdr);

struct _DumpDevice;

typedef struct _DumpFile {
	struct _DumpDevice	*device;
	const char		*name;
	int			 type;
	int			 fd;
	const char		*dir;
	off_t			 data_offset;
	off_t			 page_desc_offset_base;
	off_t			 page_desc_offset;
	darray_elfphdr		*loadsegments;
	struct new_utsname	*utsname;
	darray_mapent		*map;
} DumpFile;

DumpFile	*open_dump_file(char *dir);
DumpFile	*open_vmcore(const char *name, int flag);
int		 rename_dump_file(DumpFile *);
int		 read_page_desc(DumpFile *, page_desc_t *);
int		 write_page_desc(DumpFile *, page_desc_t *);
int		 read_page_data(DumpFile *, void *data, page_desc_t *);
int		 write_page_data(DumpFile *, const void *data, page_desc_t *);
int		 dump_read(DumpFile *, void *data, uint32_t size);
int		 dump_write(DumpFile *, const void *data, uint32_t size);
int		 dump_pread(DumpFile *dump_file, void *data, uint32_t size, off_t off);
void		*alloc_blocks(DumpFile *, int block_nr);
int		 read_blocks(DumpFile *, void *data, int block_nr);
int		 write_blocks(DumpFile *, const void *data, int block_nr);
void		 set_data_offset(DumpFile *);
void		 set_page_desc_offset(DumpFile *, unsigned long idx);
void		 calc_data_offset(DumpFile *);
int		 check_vmcore_header(DumpFile *);
int		 read_vmcore_structs(DumpFile *);
void		 show_header_info(struct disk_dump_header *, bool verbose);
void		 show_dump_info(struct disk_dump_header *, bool verbose);

static inline int dump_seek(DumpFile *dump_file, off_t off) {
	if (lseek(dump_file->fd, off, SEEK_SET) != off)
		return -1;

	return 0;
}

#endif
