#ifndef DUMPHEADER_H
#define DUMPHEADER_H
/*
 * Dump header that doesn't depend on the architecture
 */
/* $Id: dumpheader.h,v 1.18 2007/05/14 21:58:20 akira Exp $ */

#include <linux/utsname.h>
#include "arch/dumpheader.h"
#include "diskdumpcmn.h"

#define DUMP_HEADER_COMPLETED	0
#define DUMP_HEADER_INCOMPLETED	1
#define DUMP_HEADER_IN_PROGRESS	2
#define DUMP_HEADER_SHORT_AREA	4
#define DUMP_HEADER_COMPRESSED	8

/*
 * Never change the following structure
 * because kernel refers to it so as to dump
 */
struct disk_dump_header {
	char			signature[DUMP_SIG_LEN];/* = "DISKDUMP" */
	int			header_version;	/* Dump header version */
	struct new_utsname	utsname;	/* copy of system_utsname */
	struct timeval		timestamp;	/* Time stamp */
	unsigned int		status;		/* Above flags */
	int			block_size;	/* Size of a block in byte */
	int			sub_hdr_size;	/* Size of arch dependent
						   header in blocks */
	unsigned int		bitmap_blocks;	/* Size of Memory bitmap in
						   block */
	unsigned int		max_mapnr;	/* = max_mapnr */
	unsigned int		total_ram_blocks;/* Number of blocks should be
						   written */
	unsigned int		device_blocks;	/* Number of total blocks in
						 * the dump device */
	unsigned int		written_blocks;	/* Number of written blocks */
	unsigned int		current_cpu;	/* CPU# which handles dump */
	int			nr_cpus;	/* Number of CPUs */
	struct task_struct	*tasks[];
};

/* Header associated to each physical page of memory saved in the system
 * crash dump.
 */
struct dump_page_header {
	unsigned long long	page_flags;	/* Page flags */
	unsigned int		size;		/* Size */
	unsigned int		flags;		/* Dump flags */
};

typedef unsigned long long pageflags_t;

typedef struct page_desc {
	off_t			offset;		/* the offset to the page data*/
	unsigned int		size;		/* the size of this dump page */
	unsigned int		flags;		/* flags */
	pageflags_t		page_flags;	/* page flags */
} page_desc_t;

#define DUMP_DH_COMPRESSED	0x1	/* page is compressed */
#define DUMP_DH_MAPPING_ANON	0x2	/* page is mapped as anonymous */

#define DumpPageCompressed(pg)		((pg).flags & DUMP_DH_COMPRESSED)
#define DumpPageMappingAnon(pg)		((pg).flags & DUMP_DH_MAPPING_ANON)

#endif /* DUMPHEADER_H */
