#ifndef SAVECORE_H
#define SAVECORE_H

/* $Id: savecore.h,v 1.13 2007/04/27 22:01:13 tachino Exp $ */

#include "diskdumpelf.h"
#include "dumpdevice.h"

/* Save dump into "/var/crash/127.0.0.1-<date-time>/vmcore[-incomplete]" */
#define DISKDUMP_DIR_PREFIX	"/var/crash/"
#define DISKDUMP_IP		0x7f000001

#define COMPLETE_DUMP_FILE	"vmcore"
#define INCOMPLETE_DUMP_FILE	COMPLETE_DUMP_FILE"-incomplete"	/* default */

/* Available only once when there are not enough space to dump memory */
#define DISKDUMP_NOSPACE	DISKDUMP_DIR_PREFIX"scripts/diskdump-nospace"

/* Available only once after a dump file is created */
#define DISKDUMP_SUCCESS	DISKDUMP_DIR_PREFIX"scripts/diskdump-success"

/* The minimum size of memory hole which causes splitting segments */
#define HOLE_SIZE	256

/* Utility macros */
#define roundup(x, y)		((((x) + ((y) - 1)) / (y)) * (y))
#define rounddown(x, y)		(((x) / (y)) * (y))
#define divideup(x, y)		(((x) + ((y) - 1)) / (y))


/* Error code for several function defined in savecore and dumpfilter */
enum {
	COMPLETE,			/* 0 (not an error) */
	BLOCK_SIZE_INVALID,
	SEEK_ERROR_IN_DUMP_HEADER,
	READ_ERROR_IN_DUMP_HEADER,
	SEEK_ERROR_IN_DUMP_SUB_HEADER,
	READ_ERROR_IN_DUMP_SUB_HEADER,
	SEEK_ERROR_IN_MEMORY_BITMAP,
	READ_ERROR_IN_MEMORY_BITMAP,
	NO_PANIC_DUMP_HEADER,
	BLOCK_SIZE_NOT_MATCH,
	NUMBER_OF_CPUS_INVALID,
	SEEK_ERROR_IN_DUMP_DEVICE,
	READ_ERROR_IN_DUMP_DEVICE,
	SEEK_ERROR_IN_DUMP_FILE,
	READ_ERROR_IN_DUMP_FILE,
	WRITE_ERROR_IN_DUMP_FILE,
	DUMP_FILE_SIZE_UNDETERMINED,
	INVALID_SIZE_IN_PAGE_HEADER,
	UNCOMPRESS_ERROR,
	WRITE_ERROR_IN_DUMP_HEADER,
	WRITE_ERROR_IN_DUMP_SUB_HEADER,
	READ_ERROR_IN_PAGE_DESC,
	READ_ERROR_IN_PAGE_DATA,
	WRITE_ERROR_IN_PAGE_DESC,
	WRITE_ERROR_IN_PAGE_DATA,
	INVALID_ELF_SEGMENTS,
	UTSNAME_NOT_FOUND,
	FILE_OPEN_ERROR,
	SYMBOL_ERROR,
	INVALID_ADDRESS,
	LOG_BUF_TOO_LARGE,
};

static inline int dump_is_partial(const struct disk_dump_header *header)
{
	return header->bitmap_blocks >=
		divideup(divideup(header->max_mapnr, 8), header->block_size) * 2;
}

static inline bool page_is_dumpable(unsigned int nr, DumpDevice *device)
{
	return device->dumpable_bitmap[nr>>3] & (1 << (nr & 7));
}

static inline void page_dumpable_clear(unsigned int nr, DumpDevice *device)
{
	device->dumpable_bitmap[nr>>3] &= ~(1 << (nr & 7));
}

static inline char *get_dumpable_bitmap(DumpDevice *device)
{
	return device->bitmap + device->bitmap_len / 2;
}

#endif
