/*
 *  Copyright (C) 2007  FUJITSU LIMITED
 */
/* $Id: systemmap.c,v 1.3 2007/05/04 14:50:35 akira Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include "systemmap.h"
#include "darray_string.h"

#define mapent_copy(src)						\
	(mapent){ .addr = src.addr, .name = xstrdup(src.name) }

#define mapent_free(src)						\
	free(src.name)

DEFINE_DARRAY_ALLOC(mapent, mapent_copy, mapent_free);

static mapent *systemmap_find(darray_mapent *map, const char *name)
{
	mapent *ent;

	darray_foreach(ent, map)
		if (strcmp(ent->name, name) == 0)
			return ent;

	return NULL;
}

addr_t systemmap_vaddr(darray_mapent *map, const char *name)
{
	mapent *ent = systemmap_find(map, name);

	return ent ? ent->addr : (addr_t)(-1L);
}

int systemmap_fill(darray_mapent *map, const char *filename)
{
	FILE *fp;
	char line[BUFSIZ];
	mapent *mapent;
	darray_string *da;

	if ((fp = fopen(filename, "r")) == NULL)
		return FAILED;

	while (fgets(line, sizeof(line), fp) != NULL) {
		da = split(line, NULL, 0);
		if (da->length != 3)
			continue;
		if ((mapent = systemmap_find(map, da->ary[2])) != NULL)
			mapent->addr = strtoul(da->ary[0], 0, 16);

		darray_string_free(da);
	}

	fclose(fp);
	return SUCCEEDED;
}
